/*
 * Copyright (c) 2020-2021, Bluetrum Development Team
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Change Logs:
 * Date           Author       Notes
 * 2021/01/21     greedyhao    The first version
 */

#include <rtthread.h>

#if ((defined (BSP_USING_SDCARD)) && (defined (RT_USING_DFS_ROMFS)) )
////////////////////////////////////////////////////////////////

#include <dfs_fs.h>
#include "dfs_romfs.h"

int ab32_romfs_mount(void)
{
   // return 0;

    if (dfs_mount(RT_NULL, "/", "rom", 0, &(romfs_root)) == 0)
    {
        rt_kprintf("ROM file system initializated!\n");
    }
    else
    {
        rt_kprintf("ROM file system initializate failed!\n");
    }

    return 0;
}
INIT_ENV_EXPORT(ab32_romfs_mount);

////////////////////////////////////////////////////////////////
#include <dfs_elm.h>
#include <dfs_fs.h>
#include <dfs_posix.h>
#include "drv_gpio.h"

#define DRV_DEBUG
#define DBG_LVL DBG_LOG
#define DBG_TAG "app.card"
#include <rtdbg.h>

void sd_mount(void *parameter)
{
    while (1)
    {
        rt_thread_mdelay(2000);
        if(rt_device_find("sd0") != RT_NULL)
        {
            if (dfs_mount("sd0", "/sd0", "elm", 0, 0) == RT_EOK)
            {
                LOG_I("sd card mount to '/sd0' OK\r\n");
                break;
            }
            else
            {
                LOG_W("sd card mount to '/sd0' failed!\r\n");
            }
        }
    }
}

int ab32_sdcard_mount(void)
{
    rt_thread_t tid;

    tid = rt_thread_create("sd_mount", sd_mount, RT_NULL,
            1024*4, RT_THREAD_PRIORITY_MAX - 2, 20);
    if (tid != RT_NULL)
    {
        rt_thread_startup(tid);
    }
    else
    {
        LOG_E("create sd_mount thread err!\r\n");
    }
    return RT_EOK;
}
INIT_APP_EXPORT(ab32_sdcard_mount);

//

#elif defined (BSP_USING_SDCARD)

#include <dfs_elm.h>
#include <dfs_fs.h>
#include <dfs_posix.h>
#include "drv_gpio.h"

#define DRV_DEBUG
#define DBG_TAG "app.card"
#include <rtdbg.h>

void sd_mount(void *parameter)
{
    while (1)
    {
        rt_thread_mdelay(15000);
        if(rt_device_find("sd0") != RT_NULL)
        {
            if (dfs_mount("sd0", "/", "elm", 0, 0) == RT_EOK)
            {
                LOG_I("sd card mount to '/'\r\n");
                break;
            }
            else
            {
                LOG_W("sd card mount to '/' failed!\r\n");
            }
        }
    }
}

int ab32_sdcard_mount(void)
{
    rt_thread_t tid;

    tid = rt_thread_create("sd_mount", sd_mount, RT_NULL,
            1024, RT_THREAD_PRIORITY_MAX - 2, 20);
    if (tid != RT_NULL)
    {
        rt_thread_startup(tid);
    }
    else
    {
        LOG_E("create sd_mount thread err!\r\n");
    }
    return RT_EOK;
}
INIT_APP_EXPORT(ab32_sdcard_mount);
#elif defined (RT_USING_DFS_ROMFS)

#include <dfs_fs.h>
#include "dfs_romfs.h"

int ab32_romfs_mount(void)
{
   // return 0;

    if (dfs_mount(RT_NULL, "/", "rom", 0, &(romfs_root)) == 0)
    {
        rt_kprintf("ROM file system initializated!\n");
    }
    else
    {
        rt_kprintf("ROM file system initializate failed!\n");
    }

    return 0;
}
INIT_ENV_EXPORT(ab32_romfs_mount);
#endif

#ifdef RT_USING_DFS_RAMFS
//debug
#define DBG_LVL DBG_LOG
#define DBG_TAG "RamFs"
#include <rtdbg.h>

//head

#include <dfs_fs.h>
//
#include <rtthread.h>
#include <dfs.h>
#include <dfs_fs.h>
#include <dfs_file.h>

#include "dfs_ramfs.h"

int mnt_ram_elminit(void)
{
    rt_uint8_t *ramfs_pool = RT_NULL;
    struct dfs_ramfs *ramfs = NULL;
    rt_size_t size = 2 * 1024;
    ramfs_pool = rt_malloc(size);
    if (!ramfs_pool)
    {
        LOG_W("ramfs_pool = rt_malloc(size); Failed !");
        return RT_ERROR;
    }LOG_I("ramfs_pool OK");
    //
    ramfs = dfs_ramfs_create(ramfs_pool, size);
    if (NULL == ramfs)
    {
        LOG_W("dfs_ramfs_create Failed !");
        // return RT_ERROR;
    }LOG_I("dfs_ramfs_create OK");
    //
    //必须挂载到根目录而且不能创建文件夹,否则报错
    if (dfs_mount(RT_NULL, "/", "ram", 0, ramfs) == 0)
    {
        LOG_I("ram file system initializated;\n");
        return RT_EOK;
    }
    else
    {
        LOG_W("ram file system initializate failed;\n");
        return RT_ERROR;
    }
}
INIT_ENV_EXPORT(mnt_ram_elminit);

//MSH_CMD_EXPORT(mnt_ram_elminit, rams mount);

#endif

